@@ -3,8 +3,8 @@
  *
  *   (C) 2003 Dominic Morris
  *
- *   $Id: app.c,v 1.31 2003/11/27 22:47:58 dom Exp $
- *   $Date: 2003/11/27 22:47:58 $
+ *   $Id: app.c,v 1.34 2003/11/30 23:06:20 dom Exp $
+ *   $Date: 2003/11/30 23:06:20 $
  *
  *
  *   Implements the application
@@ -58,8 +58,8 @@ typedef struct {
 
 static char         *c_livetv_file = NULL;
 static char         *c_liveradio_file = NULL;
-static char         *c_mp3_dir     = NULL;
-static char         *c_mpeg_dir    = NULL;
+static char         *c_mp3_dir     = ".";
+static char         *c_mpeg_dir    = ".";
 
 static int           livetv_num    = 0;
 static channel_t    *livetv        = NULL;
@@ -81,8 +81,11 @@ static int           liveradio_keys(void *ptr, int code);
 static int           liveradio_ack(int acktype, void *param, unsigned char *buf, int len);
 
 
-static int           app_recordings(menu_t *menu,void *iapp, int sel);
-
+static int           app_mpeg(menu_t *menu,void *iapp, int sel);
+static int           process_mpeg(menu_t *menu,void *iapp, int sel);
+static int           mpeg_keys(void *ptr, int code);
+static int           mpeg_ack(int acktype, void *param, unsigned char *buf, int len);
+static int           mpeg_colour_keys(menu_t *menu, void *iapp, int key, int sel);
 
 static int           app_music(menu_t *menu,void *iapp, int sel);
 static int           process_music(menu_t *menu,void *iapp, int sel);
@@ -165,7 +168,7 @@ void delete_app(app_t *app)
     }
 
     if ( app->next_media ) {
-	next_media_clear(app);
+        next_media_clear(app);
     }
 
     free(app);
@@ -182,7 +185,7 @@ static int app_main(menu_t *menu,void *iapp, int sel)
     menu_set_title(app->menu,"MediaMVP");
     menu_add(app->menu,"Live TV",app_live_tv);
     menu_add(app->menu,"Live Radio",app_live_radio);
-    menu_add(app->menu,"Recordings",app_recordings);
+    menu_add(app->menu,"Recordings",app_mpeg);
     menu_add(app->menu,"Music",app_music);
     menu_add(app->menu,"Settings",app_settings);
     menu_display(app->menu);
@@ -243,8 +246,8 @@ static int livetv_ack(int acktype, void *param, unsigned char *buf, int len)
         /* The client has sent a stop message, this could be response to "Stop" key press or
            us sending out a stop so we can switch to the net media file */ 
         if ( app->num_next_media ) {
-	    next_media_pop(app);           
-	    app->paused = FALSE;
+            next_media_pop(app);           
+            app->paused = FALSE;
             gui_send_play(app->gui,app->current_media);
         } else {
             if ( ( stream_get_lasttype(app->dongle->stream) & MEDIA_MASK ) == MEDIA_MPEG ) {
@@ -255,7 +258,7 @@ static int livetv_ack(int acktype, void *param, unsigned char *buf, int len)
                 free(app->current_media);
                 app->current_media = NULL;
             }
-	    app->paused = FALSE;
+            app->paused = FALSE;
             /* Remove registered handlers */
             gui_register_keys(app->gui,REGISTER_MEDIA,NULL,app);
             gui_register_ack(app->gui,REGISTER_MEDIA,NULL,app);
@@ -330,11 +333,11 @@ static int process_liveradio(menu_t *menu,void *iapp, int sel)
     printf("Sending media do request %s\n",chan->url);
     if ( app->current_media == NULL ) {        
         app->current_media = strdup(chan->url);
-	app->paused = FALSE;
+        app->paused = FALSE;
         gui_send_play(app->gui,chan->url);
     } else {
-	next_media_clear(app);
-	next_media_push(app,chan->url);       
+        next_media_clear(app);
+        next_media_push(app,chan->url);       
         gui_send_message(app->gui,&rfb_media_stop);
     }
     gui_register_keys(app->gui,REGISTER_MEDIA,liveradio_keys,app);
@@ -350,7 +353,7 @@ static int liveradio_keys(void *ptr, int code)
     switch ( code ) { 
     case kStop:
         gui_send_message(app->gui,&rfb_media_stop);
-	next_media_clear(app);
+        next_media_clear(app);
         menu_display(app->menu);
         break;
     case kVolUp:
@@ -375,8 +378,8 @@ static int liveradio_ack(int acktype, void *param, unsigned char *buf, int len)
     switch ( acktype ) {
     case RDC_STOP:     /* The client has sent a stop message */  
         if ( app->num_next_media ) {
-	    next_media_pop(app);
-	    app->paused = FALSE;
+            next_media_pop(app);
+            app->paused = FALSE;
             gui_send_play(app->gui,app->current_media);
         } else {
             /* Remove registered handlers */
@@ -384,7 +387,7 @@ static int liveradio_ack(int acktype, void *param, unsigned char *buf, int len)
                 free(app->current_media);
                 app->current_media = NULL;
             }
-	    app->paused = FALSE;
+            app->paused = FALSE;
             gui_register_keys(app->gui,REGISTER_MEDIA,NULL,app);
             gui_register_ack(app->gui,REGISTER_MEDIA,NULL,app);
         }
@@ -397,23 +400,207 @@ static int liveradio_ack(int acktype, void *param, unsigned char *buf, int len)
 }
 
 
-
-
-
-
-static int app_recordings(menu_t *menu,void *iapp, int sel)
+/*
+ * Handle menus for mpeg files 
+ */
+static int app_mpeg(menu_t *menu,void *iapp, int sel)
 {
-    app_t     *app = iapp;
-    menu_t    *child;
-    child = new_menu(app->dongle,app->render,menu,app,0);
-    menu_set_title(child,"Video Recordings");  
+    app_t      *app = iapp;
+    dirlist_t  *dirlist;
+    menu_t     *child;
+    char        buf[256];
+    int         i;
 
+    dirlist = new_dirlist(app);
+
+    dirlist->path = strdup(c_mpeg_dir);
+
+    /* Scan the dirlist */
+    dirlist->filelist = directory_scan(c_mpeg_dir,&dirlist->filelist_num);
+
+    child = new_menu(app->dongle,app->render,menu,dirlist,0);
+    menu_set_title(child,"Video Recordings");   
+    menu_set_app_clearup(child,delete_dirlist);
+
+    for ( i = 0; i < dirlist->filelist_num; i++ ) {
+        if ( dirlist->filelist[i].flags ) {
+            snprintf(buf,sizeof(buf),"[%s]",dirlist->filelist[i].name);
+        } else {
+            snprintf(buf,sizeof(buf),"%s",dirlist->filelist[i].name);
+        }       
+        menu_add(child,buf,process_mpeg);
+    }
+    menu_set_colour_actions(child,"Back",NULL,NULL,NULL,mpeg_colour_keys);
 
     menu_display(child);
+    return 1;  
+}
+
+static int process_mpeg(menu_t *menu,void *iapp, int sel)
+{
+    char         buf[FILENAME_MAX+1];
+    dirlist_t    *dirlist = iapp;
+    dirlist_t    *dirlist2;
+    filelist_t   *entry;
+    menu_t       *child;
+    int           i;
+
+
+    entry = &dirlist->filelist[sel];
+
+    if ( entry->flags ) {
+        /* The selection was a directory - we need to create a new menu - hah! */
+        dirlist2 = new_dirlist(dirlist->app);
+
+        /* Scan the dirlist */
+        snprintf(buf,sizeof(buf),"%s/%s",dirlist->path,entry->name);
+        dirlist2->path = strdup(buf);
+        dirlist2->filelist = directory_scan(buf,&dirlist2->filelist_num);
+
+        child = new_menu(dirlist2->app->dongle,dirlist2->app->render,menu,dirlist2,0);
+        snprintf(buf,sizeof(buf),"Video - %s",entry->name);
+        menu_set_title(child,buf);   
+        menu_set_app_clearup(child,delete_dirlist);
+            
+        for ( i = 0; i < dirlist2->filelist_num; i++ ) {
+            if ( dirlist2->filelist[i].flags ) {
+                snprintf(buf,sizeof(buf),"[%s]",dirlist2->filelist[i].name);
+            } else {
+                snprintf(buf,sizeof(buf),"%s",dirlist2->filelist[i].name);
+            }       
+            menu_add(child,buf,process_mpeg);
+        }
+        menu_set_colour_actions(child,"Back",NULL,NULL,NULL,music_colour_keys);
+
+        menu_display(child);
+        return 1;        
+    } else {
+        snprintf(buf,sizeof(buf),"%s/%s",dirlist->path,entry->name);
+        if ( dirlist->app->current_media == NULL ) {        
+            dirlist->app->current_media = strdup(buf);
+            dirlist->app->paused = FALSE;
+//            next_media_push(dirlist->app,buf);
+            gui_send_play(dirlist->app->gui,buf);
+        } else {
+            next_media_clear(dirlist->app);
+            next_media_push(dirlist->app,buf);           
+            dirlist->app->paused = FALSE;
+            gui_send_message(dirlist->app->gui,&rfb_media_stop);    
+        }
+        gui_register_keys(dirlist->app->gui,REGISTER_MEDIA,mpeg_keys,dirlist->app);
+        gui_register_ack(dirlist->app->gui,REGISTER_MEDIA,mpeg_ack,dirlist->app);
+    }  
+
+    return 1;     /* Carry on displaying this menu */
+}
+
+static int mpeg_colour_keys(menu_t *menu, void *iapp, int key, int sel)
+{
+    switch ( key ) {
+    case kRed:
+        return 0;   
+    default:
+    }
+
+    return 1;     /* Carry on displaying this menu */
+}
+
+static int mpeg_keys(void *ptr, int code)
+{
+    app_t     *app = ptr;
+
+    switch ( code ) { 
+    case kStop:
+        next_media_clear(app);
+        gui_send_message(app->gui,&rfb_media_stop);
+        menu_display(app->menu);
+        break;
+    case kPlay:
+        gui_send_play(app->gui,app->current_media);
+        app->paused = FALSE;
+        break;
+    case kPause:
+        if ( app->paused ) {
+            app->paused = FALSE;
+            gui_send_play(app->gui,app->current_media);
+        } else {
+            app->paused = TRUE;
+            gui_send_message(app->gui,&rfb_media_pause);
+        }
+        break;
+    case kVolUp:
+        gui_send_message(app->gui,&rfb_media_volup);
+        break;
+    case kVolDn:
+        gui_send_message(app->gui,&rfb_media_voldown);
+        break;
+    case kMute:
+        gui_send_message(app->gui,&rfb_media_mute);
+        break;
+#if 0
+        /* Forward/reverse don't work...erk! */
+    case kFastFwd:
+        gui_send_message(app->gui,&rfb_media_forward);
+        break;
+    case kFastRew:
+        gui_send_message(app->gui,&rfb_media_rewind);
+        break;
+#endif
+    case kSkip:
+        gui_send_message(app->gui,&rfb_media_stop);
+        break;
+    default:
+        return 0;      /* Carry on processing menu keys */
+    }   
     return 1;
 }
 
 
+static int mpeg_ack(int acktype, void *param, unsigned char *buf, int len)
+{
+    app_t         *app = param;
+
+    switch ( acktype ) {
+    case RDC_PLAY:
+        if ( (stream_get_type(app->dongle->stream) & MEDIA_MASK) == MEDIA_MPEG ) {
+            gui_send_message(app->gui,&rfb_media_menu);
+            gui_send_message(app->gui,&rfb_display_off);
+        }
+        break;
+    case RDC_STOP:     /* The client has sent a stop message */ 
+        if ( app->num_next_media ) {
+            next_media_pop(app);
+            app->paused = FALSE;
+            gui_send_play(app->gui,app->current_media);          
+        } else {
+          if ( ( stream_get_lasttype(app->dongle->stream) & MEDIA_MASK ) == MEDIA_MPEG ) {
+                gui_send_message(app->gui,&rfb_media_menu);
+                gui_send_message(app->gui,&rfb_display_on);
+            }
+            if ( app->current_media ) {
+                free(app->current_media);
+                app->current_media = NULL;
+            }
+            app->paused = FALSE;
+            /* Remove registered handlers */
+            gui_register_keys(app->gui,REGISTER_MEDIA,NULL,app);
+            gui_register_ack(app->gui,REGISTER_MEDIA,NULL,app);            
+        }
+        break;
+    case RDC_FORWARD:
+    case RDC_REWIND:
+        gui_send_play(app->gui,app->current_media);
+        break;
+    default:
+        return 0;    /* Kick it up a level */
+    }
+
+    return 1;        /* Don't process anymore */
+}
+
+
+
+
 /*
  *   Handle the root menu for Music menu
  */
@@ -485,20 +672,20 @@ static int process_music(menu_t *menu,void *iapp, int sel)
             }       
             menu_add(child,buf,process_music);
         }
-	menu_set_colour_actions(child,"Back","Play dir",NULL,NULL,music_colour_keys);
+        menu_set_colour_actions(child,"Back","Play dir",NULL,NULL,music_colour_keys);
 
         menu_display(child);
         return 1;        
     } else {
-	snprintf(buf,sizeof(buf),"%s/%s",dirlist->path,entry->name);
+        snprintf(buf,sizeof(buf),"%s/%s",dirlist->path,entry->name);
         if ( dirlist->app->current_media == NULL ) {        
             dirlist->app->current_media = strdup(buf);
-	    dirlist->app->paused = FALSE;
+            dirlist->app->paused = FALSE;
             gui_send_play(dirlist->app->gui,buf);
         } else {
-	    next_media_clear(dirlist->app);
-	    next_media_push(dirlist->app,buf);           
-	    dirlist->app->paused = FALSE;
+            next_media_clear(dirlist->app);
+            next_media_push(dirlist->app,buf);           
+            dirlist->app->paused = FALSE;
             gui_send_message(dirlist->app->gui,&rfb_media_stop);    
         }
         gui_register_keys(dirlist->app->gui,REGISTER_MEDIA,music_keys,dirlist->app);
@@ -517,30 +704,30 @@ static int music_colour_keys(menu_t *menu, void *iapp, int key, int sel)
 
     switch ( key ) {
     case kRed:
-	return 0;
+        return 0;
     case kGreen:       /* Add all contents of the directory */
-	if ( dir->app->current_media ) {
-	    play = FALSE;
-	}
-	next_media_clear(dir->app);
+        if ( dir->app->current_media ) {
+            play = FALSE;
+        }
+        next_media_clear(dir->app);
 
-	for ( i = 0; i < dir->filelist_num; i++ ) {
-	    if ( dir->filelist[i].flags == 0 ) {
-		snprintf(buf,sizeof(buf),"%s/%s",dir->path,dir->filelist[i].name);
-		next_media_push(dir->app,buf);
-	    }
-	}
-	if ( dir->app->num_next_media ) {
-	    if ( play ) {
-		next_media_pop(dir->app);
-		gui_send_play(dir->app->gui,dir->app->current_media);
-		gui_register_keys(dir->app->gui,REGISTER_MEDIA,music_keys,dir->app);
-		gui_register_ack(dir->app->gui,REGISTER_MEDIA,music_ack,dir->app);
-	    } else {
-		gui_send_message(dir->app->gui,&rfb_media_stop);
-	    }
-	}
-	break;
+        for ( i = 0; i < dir->filelist_num; i++ ) {
+            if ( dir->filelist[i].flags == 0 ) {
+                snprintf(buf,sizeof(buf),"%s/%s",dir->path,dir->filelist[i].name);
+                next_media_push(dir->app,buf);
+            }
+        }
+        if ( dir->app->num_next_media ) {
+            if ( play ) {
+                next_media_pop(dir->app);
+                gui_send_play(dir->app->gui,dir->app->current_media);
+                gui_register_keys(dir->app->gui,REGISTER_MEDIA,music_keys,dir->app);
+                gui_register_ack(dir->app->gui,REGISTER_MEDIA,music_ack,dir->app);
+            } else {
+                gui_send_message(dir->app->gui,&rfb_media_stop);
+            }
+        }
+        break;
     default:
     }
 
@@ -553,22 +740,23 @@ static int music_keys(void *ptr, int code)
 
     switch ( code ) { 
     case kStop:
+        next_media_clear(app);
         gui_send_message(app->gui,&rfb_media_stop);
         menu_display(app->menu);
         break;
     case kPlay:
-	gui_send_play(app->gui,app->current_media);
-	app->paused = FALSE;
-	break;
+        gui_send_play(app->gui,app->current_media);
+        app->paused = FALSE;
+        break;
     case kPause:
-	if ( app->paused ) {
-	    app->paused = FALSE;
-	    gui_send_play(app->gui,app->current_media);
-	} else {
-	    app->paused = TRUE;
-	    gui_send_message(app->gui,&rfb_media_pause);
-	}
-	break;
+        if ( app->paused ) {
+            app->paused = FALSE;
+            gui_send_play(app->gui,app->current_media);
+        } else {
+            app->paused = TRUE;
+            gui_send_message(app->gui,&rfb_media_pause);
+        }
+        break;
     case kVolUp:
         gui_send_message(app->gui,&rfb_media_volup);
         break;
@@ -579,14 +767,14 @@ static int music_keys(void *ptr, int code)
         gui_send_message(app->gui,&rfb_media_mute);
         break;
     case kFastFwd:
-	gui_send_message(app->gui,&rfb_media_forward);
-	break;
+        gui_send_message(app->gui,&rfb_media_forward);
+        break;
     case kFastRew:
-	gui_send_message(app->gui,&rfb_media_rewind);
-	break;
+        gui_send_message(app->gui,&rfb_media_rewind);
+        break;
     case kSkip:
-	gui_send_message(app->gui,&rfb_media_stop);
-	break;
+        gui_send_message(app->gui,&rfb_media_stop);
+        break;
     default:
         return 0;      /* Carry on processing menu keys */
     }   
@@ -600,8 +788,8 @@ static int music_ack(int acktype, void *param, unsigned char *buf, int len)
     switch ( acktype ) {
     case RDC_STOP:     /* The client has sent a stop message */  
         if ( app->num_next_media ) {
-	    next_media_pop(app);
-	    app->paused = FALSE;
+            next_media_pop(app);
+            app->paused = FALSE;
             gui_send_play(app->gui,app->current_media);          
         } else {
             /* Remove registered handlers */
@@ -609,15 +797,15 @@ static int music_ack(int acktype, void *param, unsigned char *buf, int len)
                 free(app->current_media);
                 app->current_media = NULL;
             }
-	    app->paused = FALSE;
+            app->paused = FALSE;
             gui_register_keys(app->gui,REGISTER_MEDIA,NULL,app);
             gui_register_ack(app->gui,REGISTER_MEDIA,NULL,app);
         }
         break;
     case RDC_FORWARD:
     case RDC_REWIND:
-	gui_send_play(app->gui,app->current_media);
-	break;
+        gui_send_play(app->gui,app->current_media);
+        break;
     default:
         return 0;    /* Kick it up a level */
     }
@@ -660,7 +848,7 @@ static int app_settings(menu_t *menu,void *iapp, int sel)
     }
 
     menu_add_option(child,"Flicker Control:",process_settings,
-                    MENU_INT,3,flicker,&app->dongle->flickermode);
+                    MENU_INT,4,flicker,&app->dongle->flickermode);
 
     menu_add_option(child,"Aspect Ratio:",process_settings,
                     MENU_INT,2,aspect,&app->dongle->aspectratio);
@@ -879,12 +1067,12 @@ static void next_media_clear(app_t *app)
     int         i;
 
     if ( app->num_next_media ) {
-	for ( i = 0; i > app->num_next_media; i++ ) {
-	    free(app->next_media[i]);
-	}
-	free(app->next_media);
-	app->next_media = NULL;
-	app->num_next_media = 0;
+        for ( i = 0; i > app->num_next_media; i++ ) {
+            free(app->next_media[i]);
+        }
+        free(app->next_media);
+        app->next_media = NULL;
+        app->num_next_media = 0;
     }
 }
 
@@ -893,19 +1081,19 @@ static void next_media_pop(app_t *app)
     int       i;
 
     if ( app->num_next_media ) {     /* Which it already is... */
-	if ( app->current_media ) {
-	    free(app->current_media);
-	}
+        if ( app->current_media ) {
+            free(app->current_media);
+        }
 
-	app->current_media = app->next_media[0];
+        app->current_media = app->next_media[0];
 
-	for ( i = 1; i < app->num_next_media; i++ ) {
-	    app->next_media[i-1] = app->next_media[i];
-	}
-	if ( --app->num_next_media == 0 ) {
-	    free(app->next_media);
-	    app->next_media = NULL;
-	}
+        for ( i = 1; i < app->num_next_media; i++ ) {
+            app->next_media[i-1] = app->next_media[i];
+        }
+        if ( --app->num_next_media == 0 ) {
+            free(app->next_media);
+            app->next_media = NULL;
+        }
     } 
 }
 
