@@ -3,8 +3,8 @@
  *
  *   (C) 2003 Dominic Morris
  *
- *   $Id: gui.c,v 1.25 2003/11/27 22:47:58 dom Exp $
- *   $Date: 2003/11/27 22:47:58 $
+ *   $Id: gui.c,v 1.26 2003/11/28 00:24:10 dom Exp $
+ *   $Date: 2003/11/28 00:24:10 $
  *
  *
  *   Handles the GUI...
@@ -371,7 +371,10 @@ static int rfb_running(gui_t *gui, unsigned char *buf, int len)
                         /* Save those settings */
                         gui->dongle->tvmode = buf[11];
                         /* output mode seems to be supplied with -1 - quite why I've no idea! */
-                        gui->dongle->videooutput = buf[12] - 1;
+                        gui->dongle->videooutput = buf[12];
+			if ( dongle_version < 21330 ) {
+			    gui->dongle->videooutput--;
+			}
                         gui->dongle->flickermode = buf[13];
                         gui->dongle->aspectratio = buf[14];
                         printf("TVmode %d output = %d flicker = %d aspect = %d\n",
@@ -659,8 +662,8 @@ void gui_send_settings(gui_t *gui, int subcommand)
     gui_send_message(gui,&rfb_media_settings);
 
    
-    buf[0] = subcommand;    /* 0,1 (or 03??) */
-    buf[1] = gui->dongle->tvmode;
+    buf[0] = subcommand;   
+    buf[1] = gui->dongle->tvmode;  
     buf[2] = gui->dongle->videooutput;
     buf[3] = gui->dongle->flickermode;
     buf[4] = gui->dongle->aspectratio;
