@@ -3,8 +3,8 @@
  *
  *   (C) 2003 Dominic morris
  *
- *   $Id: gui.h,v 1.4 2003/11/19 23:48:13 dom Exp $
- *   $Date: 2003/11/19 23:48:13 $
+ *   $Id: gui.h,v 1.6 2003/11/24 14:37:30 dom Exp $
+ *   $Date: 2003/11/24 14:37:30 $
  *
  *
  *   RFB definitions
@@ -58,18 +58,7 @@ enum {
 #define RFB_SERVER_CUT_TEXT      3
 
 /* HP extensions here */
-#define RFB_MEDIA                4
-/* Some sub-commands of RFB_MEDIA and RFB_MEDIA_ACK */
-#define RFB_MEDIA_FILENAME       1
-#define RFB_MEDIA_PAUSE          2
-#define RFB_MEDIA_STOP           3
-#define RFB_MEDIA_REWIND         4
-#define RFB_MEDIA_FORWARD        5
-#define RFB_MEDIA_VOLDOWN        6
-#define RFB_MEDIA_VOLUP          7
-#define RFB_MEDIA_MPEG           8
-#define RFB_MEDIA_MUTE           9
-#define RFB_MEDIA_SETTINGS      10
+#define RFB_RDC                  4
 
 /* From client to server. */
 #define RFB_SET_PIXEL_FORMAT     0
@@ -79,7 +68,7 @@ enum {
 #define RFB_KEY_EVENT            4
 #define RFB_POINTER_EVENT        5
 #define RFB_CLIENT_CUT_TEXT      6
-#define RFB_MEDIA_ACK            7
+#define RFB_RDC_ACK            7
 
 /* HP extensions here */
 #define RFB_PING                 8
