@@ -2,8 +2,8 @@
  *  Header for MediaMVP server
  *
  *
- *  $Id: mediamvp.h,v 1.24 2003/11/27 22:47:59 dom Exp $
- *  $Date: 2003/11/27 22:47:59 $
+ *  $Id: mediamvp.h,v 1.27 2003/11/30 21:52:11 dom Exp $
+ *  $Date: 2003/11/30 21:52:11 $
  */
 
 #ifndef _MEDIAMVP_H_
@@ -28,6 +28,7 @@
 
 #include <event.h>
 
+#include "config.h"
 #include "libini.h"
 
 
@@ -76,7 +77,7 @@ typedef struct {
 
 
 extern uint32_t        main_interfaceip;
-
+extern int             dongle_version;
 
 
 /* app.c */
@@ -148,13 +149,16 @@ extern void            delete_render(render_t *render);
 extern void           *render_rgb2yvuy(render_t *render);
 extern void            render_set_fgcol(render_t *render, int red, int green, int blue);
 extern void            render_set_bgcol(render_t *render, int red, int green, int blue);
-extern void            render_print(render_t *render, int x, int y, char *text);
 extern void            render_printrow(render_t *render, int row, char *text);
 extern void            render_clear(render_t *render);
 extern int             render_get_yuvsize(render_t *render);
+extern void            render_printrow_width(render_t *render, int row, int x, int width,char *text);
 extern int             render_get_maxrows(render_t *render);
 extern int             render_get_depth(render_t *render);
 extern int             render_get_width(render_t *render);
+extern int             render_get_textspace(render_t *render, char *text);
+extern int             render_get_xoffset(render_t *render);
+extern int             render_get_yoffset(render_t *render);
 
 
 
