@@ -3,8 +3,8 @@
  *
  *   (C) 2003 Dominic Morris
  *
- *   $Id: menu.c,v 1.21 2003/11/27 22:47:59 dom Exp $
- *   $Date: 2003/11/27 22:47:59 $
+ *   $Id: menu.c,v 1.22 2003/11/30 21:52:11 dom Exp $
+ *   $Date: 2003/11/30 21:52:11 $
  *
  *
  *   Constructs a menu and all such things
@@ -52,10 +52,7 @@ struct _menu {
     void         (*app_clearup)(void *ptr);
     menu_t        *parent;
     menu_t        *child;
-    char          *red;
-    char          *green;
-    char          *yellow;
-    char          *blue;
+    char          *colours[4];   
     int          (*colour_func)(menu_t *menu,void *app, int key, int sel);
 };
 
@@ -81,7 +78,7 @@ menu_t *new_menu(dongle_t *dongle,render_t *render, menu_t *parent, void *app, i
     menu->flags   = flags;
     menu->app_clearup = NULL;
     menu->maxrows = render_get_maxrows(render);
-    menu->red = menu->green = menu->blue = menu->yellow = NULL;
+    menu->colours[0] = menu->colours[1] = menu->colours[2] = menu->colours[3] = NULL;
 
     if ( parent ) {
         menu->parent = parent;
@@ -130,23 +127,13 @@ void delete_menu(menu_t *menu)
         (menu->app_clearup)(menu->app);
     }
 
-    if ( menu->red ) {
-        free(menu->red);
+    for ( i = 0; i < 3; i++ ) {
+        if ( menu->colours[i] ) {
+            free(menu->colours[i]);
+        }
+        menu->colours[i] = NULL;
     }
 
-    if ( menu->green ) {
-        free(menu->green);
-    }
-
-    if ( menu->yellow ) {
-        free(menu->yellow);
-    }
-
-    if ( menu->blue ) {
-        free(menu->blue);
-    }
-
-
     free(menu);
 }
 
@@ -199,6 +186,8 @@ void menu_display(menu_t *menu)
     int           row;
     int           i;
     int           last;
+    int           width;
+    int           colwidth;
 
     while ( menu->child ) {
         menu = menu->child;
@@ -216,7 +205,7 @@ void menu_display(menu_t *menu)
 
     /* Calculate how many rows to print */
     last = (menu->top_offset + MENU_ROWS(menu) < menu->numoptions) 
-	? menu->top_offset + MENU_ROWS(menu) : menu->numoptions;
+        ? menu->top_offset + MENU_ROWS(menu) : menu->numoptions;
     for ( row = 2, i = menu->top_offset; i < last; i++, row++ ) {
         if ( menu->options[i].type == MENU_INT ) {
             printf("option %d val %d\n",i,*((int *)menu->options[i].value_ptr));
@@ -239,6 +228,46 @@ void menu_display(menu_t *menu)
         }
         render_printrow(render,row,buf);
     }
+
+    /* FIXME: Hardcoded hack... */
+    colwidth = (render_get_width(render) - render_get_xoffset(render) * 2) / 4;
+    for ( i = 0; i < 4; i++ ) {
+        int    offset; 
+
+        if ( menu->colours[i] ) {
+            width = render_get_textspace(render,menu->colours[i]);
+
+            if ( width > colwidth ) {
+                offset = 0;
+            } else {
+                offset = (colwidth - width) / 2;
+            }
+
+            offset += colwidth * i;
+
+            render_set_fgcol(render,BLACK);
+            switch ( i ) {
+            case 0:
+                render_set_bgcol(render,RED);
+                break;
+            case 1:
+                render_set_bgcol(render,GREEN);
+                break;
+            case 2:
+                render_set_bgcol(render,YELLOW);
+                break;
+            case 3:
+                render_set_fgcol(render,WHITE);
+                render_set_bgcol(render,BLUE);
+                break;
+            }
+            if ( menu->colours[i] ) {
+                render_printrow_width(render,menu->maxrows-1,colwidth * i,colwidth,"\n");
+                render_printrow_width(render,menu->maxrows-1,offset,colwidth,menu->colours[i]);
+            }
+        }
+    }
+        
     gui_register_keys(menu->gui,REGISTER_MENU,menu_keys,menu);
 }
 
@@ -493,42 +522,26 @@ void menu_set_colour_actions(menu_t *menu,
 			     char   *blue,
 			     int   (*func)(menu_t *menu,void *app, int key, int sel))
 {
+    int     i;
 
-    if ( menu->red ) {
-        free(menu->red);
+    for ( i = 0; i < 4; i++ ) {
+        if ( menu->colours[i] ) {
+            free(menu->colours[i]);
+        }
+        menu->colours[i] = NULL;
     }
+
     if ( red ) {
-	menu->red = strdup(red);
-    } else {
-	menu->red = NULL;
-    }
-
-    if ( menu->green ) {
-        free(menu->green);
+        menu->colours[0] = strdup(red);
     }
     if ( green ) {
-	menu->green = strdup(green);
-    } else {
-	menu->green = NULL;
+        menu->colours[1] = strdup(green);
     }
-
-    if ( menu->yellow ) {
-        free(menu->yellow);
-    } 
-
     if ( yellow ) {
-	menu->yellow = strdup(yellow);
-    } else {
-	menu->yellow = NULL;
-    }
-
-    if ( menu->blue ) {
-        free(menu->blue);
+        menu->colours[2] = strdup(yellow);
     }
     if ( blue ) {
-	menu->blue = strdup(blue);
-    } else {
-	menu->blue = NULL;
+        menu->colours[3] = strdup(blue);
     }
 
     menu->colour_func = func;
