@@ -3,8 +3,8 @@
  *
  *   (C) 2003 Dominic Morris
  *
- *   $Id: render.c,v 1.12 2003/11/27 21:28:55 dom Exp $
- *   $Date: 2003/11/27 21:28:55 $
+ *   $Id: render.c,v 1.13 2003/11/30 21:52:11 dom Exp $
+ *   $Date: 2003/11/30 21:52:11 $
  *
  *
  *   Handle rendering onto the RGB buffer stuff we've got
@@ -35,6 +35,7 @@ struct _render {
     uint8_t   updated;
 };
 
+static void     render_print(render_t *render, int x, int y, int maxx, char *text);
 static void     printchar(render_t *render,int x, int y, int *letter);
 
 render_t *new_render(int width, int depth)
@@ -181,23 +182,35 @@ void render_printrow(render_t *render, int row, char *text)
     x = OFFSET_X;
     y = row * FONT_HEIGHT + OFFSET_Y;    
 
-    render_print(render,x,y,text);
+    render_print(render,x,y,render->width - OFFSET_X,text);
+}
+
+void render_printrow_width(render_t *render, int row, int x, int width,char *text)
+{
+    int      y;
+
+    y = row * FONT_HEIGHT + OFFSET_Y;   
+
+    x += OFFSET_X;
+
+    width += x;
+
+    render_print(render,x,y,width,text);
 }
 
 
-
-void render_print(render_t *render, int x, int y, char *text)
+void render_print(render_t *render, int x, int y, int maxwidth,char *text)
 {
+    int   nbsp[FONT_HEIGHT+2] = { 1, FONT_HEIGHT, 0 };
     int  *letter;
     int   width;
 
     while ( *text ) {
         switch ( *text ) {
         case '\n':
-            letter = Font[0];
-            width = letter[0];
-            while ( x + width < ( render->width - OFFSET_X) ) {
-                printchar(render,x,y,letter);
+            width = 1;          
+            while ( x + width < maxwidth ) {
+                printchar(render,x,y,nbsp);
                 x += width;
             }
             y += FONT_HEIGHT;
@@ -210,7 +223,7 @@ void render_print(render_t *render, int x, int y, char *text)
                 letter = Font[*text - 32];
                 width = letter[0];
 
-                if ( x + width < (render->width - OFFSET_X) &&
+                if ( x + width < maxwidth &&
                      ( y < render->depth - OFFSET_Y) ) {
                     printchar(render,x,y,letter);
                     x += width;
@@ -276,6 +289,33 @@ int render_get_width(render_t *render)
     return render->width;
 }
 
+int render_get_textspace(render_t *render, char *text)
+{
+    int  *letter;
+    int   width = 0;
+
+    while ( *text ) {
+        if ( *text >= 32 ) {
+            letter = Font[*text -32];
+            width += letter[0];
+        }
+        text++;
+    }
+
+    return width;
+}
+
+int render_get_xoffset(render_t *render)
+{
+    return OFFSET_X;
+}
+
+int render_get_yoffset(render_t *render)
+{
+    return OFFSET_Y;
+}
+
+
 
 /*
  * Local Variables:
