@@ -3,8 +3,8 @@
  *
  *   (C) 2003 Dominic Morris
  *
- *   $Id: stream.c,v 1.21 2003/11/27 22:47:59 dom Exp $
- *   $Date: 2003/11/27 22:47:59 $
+ *   $Id: stream.c,v 1.23 2003/11/30 23:06:20 dom Exp $
+ *   $Date: 2003/11/30 23:06:20 $
  *
  *
  *   Handles the media stream socket..all a bit of mess at the moment..
@@ -12,31 +12,34 @@
  */
 
 #include "mediamvp.h"
+#include "ringbuffy.h"
+
+typedef struct {
+    int           fd;
+    int           id;
+    char          filename[FILENAME_MAX+1];
+    off_t         filelength;
+    ringbuffy     buffy;
+    struct event *buffy_event;
+    uint32_t      offset;              /* Should be 64 bit - check protocol */
+    uint32_t      blocklen;
+} media_t;
 
 struct _stream {
-    dongle_t      *dongle;
-    char           filename[FILENAME_MAX+1];
-    int            filename_len;
-    struct event   event;
+    dongle_t      *dongle; 
     int            fd;
-    uint32_t       offset;
-    uint32_t       blocklen;
-    int            mediaid;
-    int            mediatype;
-    int            last_mediatype;
+    struct event   event;
     char           inbuf[2048];
     int            inbufstart;
-    int            mediafd;
-    off_t          media_filelength;
-    char          *buffer;
-    int            buffer_len;
-    int            buffer_start;
-    struct event  *buffer_event;
+    media_t        media;
+    int            mediatype;
+    int            last_mediatype;
     bool_t         pending_data;
 };
 
 
 
+
 /* Definitions for the commands the MVP gives us */
 #define MEDIA_REQUEST    2
 #define MEDIA_STOP       3
@@ -54,13 +57,15 @@ static int        media_seek(stream_t *stream, unsigned char *buf, int len);
 static int        media_step(stream_t *stream, unsigned char *buf, int len);
 static int        media_8(stream_t *stream, unsigned char *buf, int len);
 
-static int        filename_parse(stream_t *stream);
+static int        filename_parse(stream_t *stream,media_t *media);
 static void       buffer_read(int fd, short event, void *arg);
 static void       stream_write_data(stream_t *stream, bool_t flush);
 
 
 struct event      listen_event;
 
+static int        c_mpeg_bufsize = 200000;
+static int        c_mp3_bufsize  = 4000;
 
 int               c_stream_port = 6337;
 static char      *c_stream_host = NULL;
@@ -104,27 +109,34 @@ static stream_t *new_stream(dongle_t *dongle)
 
     stream->dongle     = dongle;
     stream->inbufstart = 0;
-    stream->mediafd    = -1;
-    stream->buffer     = NULL;
-    stream->buffer_start = 0;
-    stream->buffer_len   = 0;
-    stream->buffer_event = NULL;
+    stream->mediatype  = 0; 
 
     return stream;
 }
 
+void delete_media(media_t *media)
+{
+    ring_destroy(&media->buffy);
+
+    if ( media->buffy_event ) {
+        event_del(media->buffy_event);
+        media->buffy_event = NULL;
+    }
+    close(media->fd);
+    shutdown(media->fd,SHUT_RDWR);
+}
+
 void delete_stream(stream_t *stream)
 {
+    /* Cleanup the media file stuff */
+    if ( stream->mediatype ) {
+        delete_media(&stream->media);
+        stream->mediatype = 0;
+    }
+    
+
+    /* Now clean up the stream input channel */
     event_del(&stream->event);
-
-    if ( stream->buffer_event ) {
-        event_del(stream->buffer_event);
-    }
-
-    if ( stream->buffer ) {
-        free(stream->buffer);
-    }
-
     close(stream->fd);
     shutdown(stream->fd,SHUT_RDWR);
     free(stream);
@@ -224,7 +236,9 @@ static void stream_read(int fd, short event, void *arg)
 
 static int media_request(stream_t *stream, unsigned char *buf, int len)
 {
+    int            filename_len;
     unsigned char *ptr;
+    media_t       *media = &stream->media;
 
     if ( len < 24 ) {
         return 0;
@@ -233,98 +247,80 @@ static int media_request(stream_t *stream, unsigned char *buf, int len)
 
     ptr = buf + 20;
 
-    BUF_TO_INT16(stream->filename_len,ptr);
+    BUF_TO_INT16(filename_len,ptr);
     ptr += 2;
 
-    if ( len < stream->filename_len + 24  ) {
+    if ( len < filename_len + 24  ) {
         return 0; /* Wait fo more data */
     }
 
-    strcpy(stream->filename,ptr);    /* Filename is NUL terminated */
+    strcpy(media->filename,ptr);    /* Filename is NUL terminated */
 
-    printf("Client has requested %s\n",stream->filename);
+    printf("Client has requested %s\n",media->filename);
 
     /* Now set up the reply string */
     /* Do a random stream id */
-    ++stream->mediaid;
+    ++media->id;
 
     /* Setup a media id thing */
     ptr = buf + 18;   
-    INT16_TO_BUF(stream->mediaid,ptr);
-
-    if (stream->mediafd != 0 ) {
-        close(stream->mediafd);
-        shutdown(stream->mediafd,SHUT_RDWR);
-        stream->mediafd = 0;
-        /* Free up buffers */
-        if ( stream->buffer != NULL ) {
-            free(stream->buffer);
-            if ( stream->buffer_event ) {
-                event_del(stream->buffer_event);
-                free(stream->buffer_event);
-                stream->buffer_event = NULL;
-            }
-            stream->buffer = NULL;
-            stream->buffer_start = 0;
-            stream->buffer_len = 0;
-        }
-    }
+    INT16_TO_BUF(media->id,ptr);
 
+    if ( stream->mediatype ) {
+        delete_media(&stream->media);
+    }  
 
     /* Open the filename now - need to handle no file existing.. */
-    filename_parse(stream);
+    filename_parse(stream,media);
 
     if ( (stream->mediatype & MEDIA_MASK) == MEDIA_MPEG ) {
+        ring_init(&media->buffy, c_mpeg_bufsize * 4);
         buf[4] = 0x01;
         buf[6] = 0xFF;
 
+        /* These parameters don't actually seem to matter... */
         buf[8] =  0x02;    /* FIXME: 720 */
-        buf[9] =  0xd0; 
+        buf[9] =  0xc0; 
         buf[10] = 0x02;    /* FIXME: 576 */
         buf[11] = 0x40;
         buf[14] = 0x09; // 0x19;    /* 6500 - like it matters... */
         buf[15] = 0xf0; // 0x64;
     } else {
+        ring_init(&media->buffy, c_mp3_bufsize * 4);
         buf[4] = 0x02;
     }
    
+    /* Set up a read event for streaming media */
+    if ( stream->mediatype & MEDIA_SOCKET ) {        
+        media->buffy_event = malloc(sizeof(*media->buffy_event));
+        event_set(media->buffy_event,media->fd,EV_READ,buffer_read,stream);
+        event_add(media->buffy_event,NULL);
+    } 
    
     write(stream->fd,buf,24);
 
-    return 24 + stream->filename_len;
+    return 24 + filename_len;
 }
 
 static int media_stop(stream_t *stream, unsigned char *buf, int len)
 {
     unsigned char *ptr;
+    media_t       *media = &stream->media;
 
     if ( len < 24 ) {
         return 0;
     }
     printf("Stop\n");
 
-    if ( stream->mediafd ) {
-        close(stream->mediafd);
-        shutdown(stream->mediafd,SHUT_RDWR);
-        stream->mediafd = 0;
-        if ( stream->buffer != NULL ) {
-            free(stream->buffer);
-            if ( stream->buffer_event ) {
-                event_del(stream->buffer_event);
-                free(stream->buffer_event);
-                stream->buffer_event = NULL;
-            }
-            stream->buffer = NULL;
-            stream->buffer_start = 0;
-            stream->buffer_len = 0;
-        }
+    if ( stream->mediatype ) {
+        delete_media(media);
     }
 
     stream->last_mediatype = stream->mediatype;
     stream->mediatype = 0;
 
     ptr = buf + 18;   
-    INT16_TO_BUF(stream->mediaid,ptr);
+    INT16_TO_BUF(media->id,ptr);
     write(stream->fd,buf,len);   
 
     return 24;
@@ -334,35 +330,24 @@ static int media_stop(stream_t *stream, unsigned char *buf, int len)
 static int media_block(stream_t *stream, unsigned char *buf, int len)
 {
     unsigned char *ptr = buf + 8;
-    int           mlen;
+    int            mlen;
+    media_t       *media = &stream->media;
 
     if ( len < 24 ) {
         return 0;
     }
 
 
-    BUF_TO_INT32(stream->blocklen,ptr);
+    BUF_TO_INT32(media->blocklen,ptr);
 
-    if (stream->buffer == NULL ) {
-        if ( stream->mediatype & MEDIA_SOCKET ) {
-            stream->buffer = malloc(stream->blocklen * 4);
-            stream->buffer_len = stream->blocklen * 4;
-            stream->buffer_start = 0;
-            stream->buffer_event = malloc(sizeof(struct event));
-            event_set(stream->buffer_event,stream->mediafd,EV_READ,buffer_read,stream);
-            event_add(stream->buffer_event,NULL);
-
-        } else {
-            stream->buffer = malloc(stream->blocklen+1);
-            stream->buffer_event = NULL;
-        }
-    }
+    /* Move this somewhere.. */
+       
 
     if ( stream->mediatype & MEDIA_SOCKET ) {
-        if ( stream->mediafd == -1 ) {
+        if ( media->fd == -1 ) {
             /* The file has come to an end */
             stream_write_data(stream,TRUE);            
-        } else if ( stream->buffer_start >= stream->blocklen ) {
+        } else if ( ring_rest(&media->buffy) >= media->blocklen ) {
             stream_write_data(stream,FALSE);
             stream->pending_data = FALSE;
         } else {
@@ -373,23 +358,25 @@ static int media_block(stream_t *stream, unsigned char *buf, int len)
     }
 
     /* It's just a file on disc, no buffering needed */
-    mlen = read(stream->mediafd,stream->buffer,stream->blocklen);
+    mlen = ring_write_file(&media->buffy,media->fd,media->blocklen);
    
-    stream->offset += mlen;
+
+    media->offset += mlen;
 
     
     ptr = buf + 8;
     INT32_TO_BUF(mlen,ptr);
-    INT32_TO_BUF(stream->offset,ptr);
+    INT32_TO_BUF(media->offset,ptr);
  
     write(stream->fd,buf,24);
-    write(stream->fd,stream->buffer,mlen);
+    ring_read_file(&media->buffy,stream->fd,mlen);
     return 24;
 }
 
 static int media_seek(stream_t *stream, unsigned char *buf, int len)
 {
     unsigned char *ptr;
+    media_t       *media = &stream->media;
 
 
     /* Needs extending slightly... */
@@ -400,8 +387,8 @@ static int media_seek(stream_t *stream, unsigned char *buf, int len)
     dump_hex(buf,24);
 
     if ( (stream->mediatype & MEDIA_SOCKET ) == 0 ) {
-	ptr = buf + 8;
-	BUF_TO_INT32(stream->offset,ptr);
+        ptr = buf + 8;
+        BUF_TO_INT32(media->offset,ptr);
     }
 
     /* Just acknowledge it back */
@@ -427,6 +414,7 @@ static int media_8(stream_t *stream, unsigned char *buf, int len)
 static int media_step(stream_t *stream, unsigned char *buf, int len)
 {
     unsigned char *ptr;
+    media_t       *media = &stream->media;
 
     /* Needs extending slightly... */
     if ( len < 24 ) {
@@ -437,23 +425,23 @@ static int media_step(stream_t *stream, unsigned char *buf, int len)
     dump_hex(buf,24);
 
     ptr = buf + 16;
-    INT32_TO_BUF(stream->offset,ptr);
+    INT32_TO_BUF(media->offset,ptr);
 
 
     if ( buf[9] == 1 ) {  /* Forward */
-	stream->offset += stream->blocklen;
+        media->offset += media->blocklen;
     } else {              /* And back - but it doesn't work at the moment */
-	stream->offset -= stream->blocklen;
+        media->offset -= media->blocklen;
     }
 
-    if ( stream->offset < 0 ) {
-	stream->offset = 0;
-    } else if ( stream->offset > stream->media_filelength ) {
-	stream->offset -= stream->blocklen;
+    if ( media->offset < 0 ) {
+        media->offset = 0;
+    } else if ( media->offset > media->filelength ) {
+        media->offset -= media->blocklen;
     }
 
     /* Nasty, possibly non portable bit... */
-    lseek(stream->mediafd,stream->offset,SEEK_SET);
+    lseek(media->fd,media->offset,SEEK_SET);
 
     write(stream->fd,buf,24);
 
@@ -462,51 +450,51 @@ static int media_step(stream_t *stream, unsigned char *buf, int len)
 
 
 
-static int filename_parse(stream_t *stream)
+static int filename_parse(stream_t *stream, media_t *media)
 {
     struct    stat sb;
     char      buffer[FILENAME_MAX+1];
     char     *url;
     int       len;
 
-    stream->media_filelength = 0;
+    media->filelength = 0;
     /* Just a quick and dirty hack.. */
-    if ( strncmp(stream->filename,"mpeg;http://",7) == 0 ) {
+    if ( strncmp(media->filename,"mpeg;http://",7) == 0 ) {
         stream->mediatype = MEDIA_MPEG|MEDIA_SOCKET;
-        stream->mediafd = http_open(stream->filename+5);
-        return stream->mediafd;
-    } else if ( strncmp(stream->filename,"mp3;http://",7) == 0 ) {
+        media->fd = http_open(media->filename+5);
+        return media->fd;
+    } else if ( strncmp(media->filename,"mp3;http://",7) == 0 ) {
         stream->mediatype = MEDIA_MP3|MEDIA_SOCKET;
-        stream->mediafd = http_open(stream->filename+4);
-        return stream->mediafd;
-    }
-      
-    if ( stat(stream->filename,&sb) >= 0 ) {
+        media->fd = http_open(media->filename+4);
+        return media->fd;
+    } 
+
+    if ( stat(media->filename,&sb) >= 0 ) {
         if ( (sb.st_mode & S_IFIFO)  ) {
             stream->mediatype = MEDIA_MPEG;
-            if ( (stream->mediafd = open(stream->filename,O_RDONLY|O_NONBLOCK) ) == -1 ) {
+            if ( (media->fd = open(media->filename,O_RDONLY|O_NONBLOCK) ) == -1 ) {
                 printf("Can't open file\n");
                 return -1;
             }
-            return stream->mediafd;
+            return stream->fd;
         }
     }
 
 
-    if ( (stream->mediafd = open(stream->filename,O_RDONLY) ) == -1 ) {
+    if ( (media->fd = open(media->filename,O_RDONLY) ) == -1 ) {
         printf("Can't open file\n");
         return -1;
     }
 
-    stream->media_filelength = sb.st_size;
+    media->filelength = sb.st_size;
 
-    len = read(stream->mediafd,buffer,sizeof(buffer) - 1 );
+    len = read(media->fd,buffer,sizeof(buffer) - 1 );
     buffer[len] = 0;
 
     /* Yes, these are meant to be assigns for MEDIA_MPEG and MEDIA_MP3 */
     if ( (strncasecmp(buffer,"mpeg",4) == 0 && (stream->mediatype = MEDIA_MPEG|MEDIA_SOCKET) ) ||
          (strncasecmp(buffer,"mp3",3) == 0 && (stream->mediatype = MEDIA_MP3|MEDIA_SOCKET) )) {
-        close(stream->mediafd);
+        close(media->fd);
         if ( (url = strchr(buffer,' ') ) == NULL )
             url = strchr(buffer,'\t');
         if ( url == NULL )
@@ -514,23 +502,24 @@ static int filename_parse(stream_t *stream)
         url++;
 
         /* Now, we should parse the url */
-        stream->mediafd = http_open(url);
+        media->fd = http_open(url);
     } else if ( strncasecmp(buffer,"mplayer",7) == 0 ) {
 #if 0
-        close(stream->mediafd);
-        stream->fd = pipe_open(stream.buffer);
+        close(smedia->fd);
+        media->fd = pipe_open(stream.buffer);
 #endif
     } else { /* It's a normal file, rewind to the start */
-        if ( strstr(stream->filename,".mpeg") || strstr(stream->filename,".mpg") || strstr(stream->filename,".MPEG") || strstr(stream->filename,".MPG") ) {
+        /* Should have some file interrogation logic here */
+        if ( strstr(media->filename,".mpeg") || strstr(media->filename,".mpg") || strstr(media->filename,".MPEG") || strstr(media->filename,".MPG") || strstr(media->filename,".vdr") || strstr(media->filename,".VDR") ) {
             stream->mediatype = MEDIA_MPEG;
         } else {
             stream->mediatype = MEDIA_MP3;
         }
 
         
-        lseek(stream->mediafd,0L,SEEK_SET);
+        lseek(media->fd,0L,SEEK_SET);
     }
-    return stream->mediafd;
+    return media->fd;
 }
 
 
@@ -539,36 +528,29 @@ static int filename_parse(stream_t *stream)
 static void buffer_read(int fd, short event, void *arg)
 {
     stream_t   *stream = arg;
+    media_t    *media  = &stream->media;
     int         r;
 
-    if ( stream->buffer_start == stream->buffer_len ) {
-        /* Buffer overflow... */
-        stream->buffer_start = stream->buffer_len - stream->blocklen;
-        memmove(stream->buffer,stream->buffer + stream->blocklen,stream->buffer_start);
-        printf("Buffer overflow, discard %d bytes\n",stream->blocklen);
-    }
-
-    r = read(fd,stream->buffer + stream->buffer_start,stream->buffer_len - stream->buffer_start );
+    r = ring_write_file(&media->buffy,fd,media->blocklen);
 
     /* Check for socket shutdown */
     if ( r == 0 || r == -1 ) {
         close(fd);
         shutdown(fd,SHUT_RDWR);
-        stream->fd = -1;
-        free(stream->buffer_event);
-        stream->buffer_event = NULL;
+        media->fd = -1;
+        free(media->buffy_event);
+        media->buffy_event = NULL;
         return;
     }
 
-    stream->buffer_start += r;
 
-    if ( stream->pending_data && stream->buffer_start >= stream->blocklen ) {
+    if ( stream->pending_data && ring_rest(&media->buffy) >= media->blocklen ) {
         stream->pending_data = FALSE;
         stream_write_data(stream,FALSE);
 
     }
 
-    event_add(stream->buffer_event,NULL);
+    event_add(media->buffy_event,NULL);
 
 }
 
@@ -578,16 +560,17 @@ static void stream_write_data(stream_t *stream, bool_t flush)
     unsigned char      msg[24];
     unsigned char     *ptr;
     int                len;
+    media_t           *media = &stream->media;
 
     if ( flush ) {
-        if ( stream->buffer_start >= stream->blocklen ) {
-            len = stream->blocklen;
+        if ( ring_rest(&media->buffy) >= media->blocklen ) {
+            len = media->blocklen;
         } else {
-            len = stream->buffer_start;
+            len = ring_rest(&media->buffy);
         }
     } else {
-        if ( stream->buffer_start >= stream->blocklen ) {
-            len = stream->blocklen;
+        if ( ring_rest(&media->buffy) >= media->blocklen ) {
+            len = media->blocklen;
         } else {
             return;  /* And wait for more data */
         }
@@ -597,20 +580,16 @@ static void stream_write_data(stream_t *stream, bool_t flush)
     memset(msg,0,24);
     msg[0] = 0x04;
     ptr = msg+6;
-    INT16_TO_BUF(stream->mediaid,ptr);
+    INT16_TO_BUF(media->id,ptr);
 
     INT32_TO_BUF(len,ptr);
-    stream->offset += len;
-    INT32_TO_BUF(stream->offset,ptr);
+    media->offset += len;
+    INT32_TO_BUF(media->offset,ptr);
 
 
     write(stream->fd,msg,24);
     if ( len ) {
-        write(stream->fd,stream->buffer,len);
-        memmove(stream->buffer,stream->buffer+len,stream->buffer_start - len);
-        stream->buffer_start -= len;
-        if ( stream->buffer_start < 0 )
-            stream->buffer_start = 0;
+        ring_read_file(&media->buffy,stream->fd,len);       
     }
 
     return;
@@ -640,6 +619,20 @@ int stream_get_lasttype(stream_t *stream)
     return ( stream->last_mediatype );
 }
 
+
+/** \brief Remux the recording so it only has one audio stream
+ *
+ *  
+ */
+
+void remux_init(media_t *media)
+{
+    /* We should read a bit of this in to find out parameters etc */
+
+}
+
+
+
 /*
  * Local Variables:
  *  indent-tabs-mode:nil
